package com.agilecontrol.nea.core.control.ejb.command;

import com.agilecontrol.nea.core.control.ejb.Command;
import com.agilecontrol.nea.core.control.event.DefaultWebEvent;
import com.agilecontrol.nea.core.control.util.ValueHolder;
import com.agilecontrol.nea.core.jcrosstab.fun.FunUtil;
import com.agilecontrol.nea.core.query.ColumnLink;
import com.agilecontrol.nea.core.query.QueryEngine;
import com.agilecontrol.nea.core.query.QueryRequestImpl;
import com.agilecontrol.nea.core.query.QuerySession;
import com.agilecontrol.nea.core.query.QueryUtils;
import com.agilecontrol.nea.core.schema.Column;
import com.agilecontrol.nea.core.schema.Table;
import com.agilecontrol.nea.core.schema.TableManager;
import com.agilecontrol.nea.core.security.SecurityUtils;
import com.agilecontrol.nea.core.security.User;
import com.agilecontrol.nea.util.NDSException;
import com.agilecontrol.nea.util.Tools;
import com.agilecontrol.nea.util.Validator;
import java.rmi.RemoteException;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

public class CxtabGetSQL extends Command {

   public ValueHolder execute(DefaultWebEvent event) throws NDSException, RemoteException {
      this.logger.debug(event.toDetailString());
      User usr = this.helper.getOperator(event);
      int cxtabId = Tools.getInt(event.getParameterValue("objectid", true), -1);
      if(!usr.getName().equals("root") && !SecurityUtils.hasObjectPermission(usr.getId().intValue(), usr.getName(), "AD_CXTAB", cxtabId, 3, event.getQuerySession())) {
         throw new NDSException("@no-permission@");
      } else {
         ValueHolder holder = new ValueHolder();
         Connection conn = QueryEngine.getInstance().getConnection();

         String message;
         try {
            message = this.prepareReport(cxtabId, usr, conn);
            holder.put("code", "0");
         } catch (Throwable var15) {
            this.logger.error("User " + usr.getName() + "@" + usr.getClientDomain() + " fail to refresh jreport:" + cxtabId, var15);
            message = "@jreport-found-error@:" + var15.getMessage();
            throw new NDSException(message);
         } finally {
            try {
               conn.close();
            } catch (Exception var14) {
               ;
            }

         }

         holder.put("message", message);
         holder.put("code", new Integer(0));
         return holder;
      }
   }

   private String prepareReport(int cxtabId, User user, Connection conn) throws Exception {
      boolean isOnHTML = false;
      QueryEngine engine = QueryEngine.getInstance();
      TableManager manager = TableManager.getInstance();
      List ed = engine.doQueryList("select ad_table_id,name from ad_cxtab where id=" + cxtabId, conn);
      int factTableId = Tools.getInt(((List)ed.get(0)).get(0), -1);
      if(factTableId == -1) {
         return null;
      } else {
         String cxtabDesc = (String)((List)ed.get(0)).get(1);
         Table factTable = manager.getTable(factTableId);
         List dimensionsH = engine.doQueryList("select columnlink, description, measure_order, hidehtml from ad_cxtab_dimension where ad_cxtab_id=" + cxtabId + " and isactive=\'Y\' and position_=\'H\' " + (isOnHTML?"and hidehtml=\'N\'":"") + " order by orderno asc", conn);
         List dimensionsV = engine.doQueryList("select columnlink, description, measure_order,hidehtml from ad_cxtab_dimension where ad_cxtab_id=" + cxtabId + " and isactive=\'Y\' and position_=\'V\' " + (isOnHTML?"and hidehtml=\'N\'":"") + " order by orderno asc", conn);
         String cxtabFilter = (String)engine.doQueryOne("select filter from ad_cxtab where id=" + cxtabId, conn);
         Locale locale = user.getLocale();
         this.logger.debug("Locale for " + user.getNameWithDomain() + "(id=" + user.getId() + ") is " + locale);
         QuerySession qsession = QueryUtils.createQuerySession(user.getId().intValue(), user.getSecurityGrade(), "", user.getLocale());
         QueryRequestImpl query = engine.createRequest(qsession);
         query.setMainTable(factTableId, true, cxtabFilter);
         int measures;
         List facts;
         ColumnLink factDescs;
         if(dimensionsH != null && dimensionsH.size() > 0) {
            for(measures = 0; measures < dimensionsH.size(); ++measures) {
               facts = (List)dimensionsH.get(measures);
               factDescs = new ColumnLink((String)facts.get(0));
               query.addSelection(factDescs.getColumnIDs(), false, (String)facts.get(1));
            }
         }

         if(dimensionsV != null && dimensionsV.size() > 0) {
            for(measures = 0; measures < dimensionsV.size(); ++measures) {
               facts = (List)dimensionsV.get(measures);
               factDescs = new ColumnLink((String)facts.get(0));
               query.addSelection(factDescs.getColumnIDs(), false, (String)facts.get(1));
            }
         }

         List var29 = engine.doQueryList("select ad_column_id, function_, userfact, description, VALUEFORMAT,valuename, param1,param2,param2 from ad_cxtab_fact where ad_cxtab_id=" + cxtabId + " and isactive=\'Y\' order by orderno asc", conn);
         ArrayList var30 = new ArrayList();
         ArrayList var31 = new ArrayList();
         boolean isDBGroupByEnabled = true;
         boolean mustBeDBGroupBy = false;

         int i;
         List mea;
         for(i = 0; i < var29.size(); ++i) {
            mea = (List)var29.get(i);
            String colId = (String)mea.get(2);
            if(Validator.isNotNull(colId)) {
               if(!colId.startsWith("=")) {
                  var30.add(colId);
                  var31.add(mea.get(3));
                  mustBeDBGroupBy = true;
               }
            } else {
               String col = (String)mea.get(1);
               int colId1 = Tools.getInt(mea.get(0), -1);
               Column col1 = TableManager.getInstance().getColumn(colId1);
               if(FunUtil.isValidGroupByFunction(col)) {
                  var30.add(FunUtil.getGroupByFunctionCall(col, col1.isVirtual()?col1.getName():factTable.getName() + "." + col1.getName()));
                  var31.add(mea.get(3));
               } else {
                  isDBGroupByEnabled = false;
               }
            }
         }

         String sql;
         if(isDBGroupByEnabled) {
            sql = query.toGroupBySQL(var30);
         } else {
            if(mustBeDBGroupBy) {
               throw new NDSException("Cxtab configuration error, found user fact(db group by function) and invalid db group by function (e.g. avg) in the same time");
            }

            for(i = 0; i < var29.size(); ++i) {
               mea = (List)var29.get(i);
               int var32 = Tools.getInt(mea.get(0), -1);
               Column var33 = TableManager.getInstance().getColumn(var32);
               if(var33 != null) {
                  query.addSelection(var32);
               }
            }

            sql = query.toSQL();
         }

         return sql;
      }
   }
}
